


def rewrite_the_footer(footer_pth, show_credit):

    text = (
        '<footer>'
            '<p id="copyright">&copy; {{ "now" | date: "%Y" }} {{ site.author.name }}</p>'
    )
    if show_credit:
        text += '<p id="credit">built with <a href="https://github.com/nvfp/now-i-can-sleep">now-i-can-sleep</a></p>'
    text += '</footer>'

    with open(footer_pth, 'w') as f:
        f.write(text)


import os
import shutil

from ..constants import TEMPLATE_DIR_PTH


def copying_template(dock):

    x = os.path.join(TEMPLATE_DIR_PTH, '_layouts')
    print(123, x)
    print(345, os.path.isdir(x))
    print('ASSETS_DIR_PTH: ', TEMPLATE_DIR_PTH)
    print('os.path.isdir(ASSETS_DIR_PTH): ', os.path.isdir(TEMPLATE_DIR_PTH))
    print('os.listdir(ASSETS_DIR_PTH): ', os.listdir(TEMPLATE_DIR_PTH))

    shutil.copytree(
        os.path.join(TEMPLATE_DIR_PTH, '_layouts'),
        os.path.join(dock, '_layouts')
    )

    shutil.copytree(
        os.path.join(TEMPLATE_DIR_PTH, '_sass'),
        os.path.join(dock, '_sass')
    )

    shutil.copytree(
        os.path.join(TEMPLATE_DIR_PTH, 'scripts'),
        os.path.join(dock, 'scripts')
    )


import os
import re
import shutil

from mykit.kit.keycrate import KeyCrate
from mykit.kit.utils import printer

from .inspect import inspect_the_container, inspect_the_dock
from .rewrite_the_header import rewrite_the_header
from .rewrite_the_footer import rewrite_the_footer
from .rewrite_jekyll_config import rewrite_jekyll_config
from .copying_template import copying_template


def run(container, dock):
    """
    ## Params
    - `container`: the nics folder bundle
    - `dock`: the branch
    """

    C_ASSETS = os.path.join(container, 'assets')
    C_TREE = os.path.join(container, 'tree')
    C_404 = os.path.join(container, '404.md')
    C_ICON = os.path.join(container, 'favicon.png')
    C_SETTINGS = os.path.join(container, 'settings.txt')

    D_HEADER = os.path.join(dock, '_includes', 'header.html')
    D_FOOTER = os.path.join(dock, '_includes', 'footer.html')
    D_ASSETS = os.path.join(dock, 'assets')
    D_404 = os.path.join(dock, '404.md')
    D_ICON = os.path.join(dock, 'favicon.png')
    D_JEKYLL_CONFIG = os.path.join(dock, '_config.yml')


    ## validate the requirements
    inspect_the_container(container)
    # inspect_the_dock()


    settings = KeyCrate(C_SETTINGS, True, True)


    ## handle init case
    if not os.path.isdir( os.path.join(dock, '_includes') ):
        os.mkdir(os.path.join(dock, '_includes'))


    rewrite_the_header(C_TREE, D_HEADER)
    rewrite_the_footer(D_FOOTER, settings.show_credit)

    rewrite_jekyll_config(D_JEKYLL_CONFIG, settings.author, settings._gh_username, settings._gh_repo)

    copying_template(dock)


    ## <rewriting 404.md>
    
    if os.path.isfile(C_404):
        printer('DEBUG: Copying 404.md.')
        text = (
            '---\n'
            'permalink: /404.html\n'
            'layout: main\n'
            'title: Page not found\n'
            '---\n\n'
        )
        with open(C_404, 'r') as f:
            text += f.read()
        
        printer(f'DEBUG: writing to {repr(D_404)}')
        with open(D_404, 'w') as f:
            f.write(text)

    ## </rewriting 404.md>


    ## <rewriting the docs>

    def rewrite_the_docs_tree_recursively(pth, base):
        for i in os.listdir(pth):

            if i == 'index.md':

                if base == '/':  # homepage
                    text = (
                        '---\n'
                        'permalink: /\n'
                        'layout: main\n'
                        'title: Home\n'
                        '---\n\n'
                    )
                    src = os.path.join( pth, 'index.md' )
                    printer(f'DEBUG: src: {repr(src)}')
                    with open(src, 'r') as f: text += f.read()
                    
                    dst = os.path.join(dock, 'index.md')
                    printer(f'DEBUG: writing to {repr(dst)}')
                    with open(dst, 'w') as f: f.write(text)
                else:
                    text = (
                        '---\n'
                        f'permalink: {base}\n'
                        'layout: main\n'
                        f"title: {list(filter(lambda s:s!='', base.split('/')))[-1]}\n"
                        '---\n\n'
                    )
                    src = os.path.join( pth, 'index.md' )
                    printer(f'DEBUG: src: {repr(src)}')
                    with open(src, 'r') as f: text += f.read()
                    
                    dst = os.path.join( dock, '_pages', os.sep.join(filter(lambda s:s!='', base.split('/'))), 'index.md')
                    printer(f'DEBUG: writing to {repr(dst)}')
                    with open(dst, 'w') as f: f.write(text)

                continue

            pth2 = os.path.join(pth, i)

            res = re.match(r'(?:\d+ -- )?(?P<name>[\w -.]+) -- (?P<url>[\w -]+)(?:.md)?', i)
            name = res.group('name')
            url = res.group('url')

            if os.path.isdir(pth2):
                
                ## if the dir not exist -> create a new one
                dir = os.path.join( dock, '_pages', os.sep.join(filter(lambda s:s!='', base.split('/'))), name )
                if not os.path.isdir(dir):
                    printer(f'DEBUG: Creating new dir: {repr(dir)}.')
                    os.mkdir(dir)

                rewrite_the_docs_tree_recursively(pth2, base+name+'/')
            else:
                text = (
                    '---\n'
                    f'permalink: {base}{url}/\n'
                    'layout: main\n'
                    f'title: {name}\n'
                    '---\n\n'
                )
                src = os.path.join( pth, i )
                printer(f'DEBUG: src: {repr(src)}')
                with open(src, 'r') as f: text += f.read()
                
                dst = os.path.join( dock, '_pages', os.sep.join(filter(lambda s:s!='', base.split('/'))), f'{name}.md')
                printer(f'DEBUG: writing to {repr(dst)}')
                with open(dst, 'w') as f: f.write(text)
    rewrite_the_docs_tree_recursively( os.path.join(container, 'tree'), '/' )

    ## </rewriting the docs>


    printer(f'INFO: start copying assets..')
    printer(f'INFO: Deleting {repr(D_ASSETS)} recursively..')
    shutil.rmtree(D_ASSETS)
    printer(f'INFO: Copying from {repr(C_ASSETS)} to {repr(D_ASSETS)}.')
    shutil.copytree(C_ASSETS, D_ASSETS)

    if os.path.isfile(C_ICON):
        printer(f'INFO: start copying favicon.png..')
        shutil.copy(C_ICON, D_ICON)


import os
import re


def rewrite_the_header(tree, header_pth):
    """
    reminder:
    - index.md will not be included in the header
    """

    header = (
        '<header>'
            '<h1>{{ page.title }}</h1>'
            '<div class="wrap">'
                '<button id="_root__nav">v Navigation</button>'
                '<div class="main" id="_root__nav-div">'
    )

    ## reminder: `base` is the X in foo.com/baseurl/{X}page
    ##           (e.g. nvfp.github.io/mykit/docs/guide/foo, then X='docs/guide/')
    def build_the_nested_divs_recursively(pth, base) -> str:
        out = ''

        for fd in sorted(os.listdir(pth)):  # reminder: the file/dir `fd` order in `pth` matters.
            if fd == 'index.md': continue  # index.md will not be shown in navigation bar
            fd_pth = os.path.join(pth, fd)

            res = re.match(r'(?:\d+ -- )?(?P<name>[\w -.]+) -- (?P<url>[\w -]+)(?:.md)?', fd)
            name = res.group('name')
            url = res.group('url')

            if os.path.isdir(fd_pth):
                out += f'<button id="{base.replace("/", "-")}{url}">> {name}</button>'  # remember to replace all slashes to hyphens
                out += f'<div class="child" id="{base.replace("/", "-")}{url}-div">'  # remember to replace all slashes to hyphens
                out += build_the_nested_divs_recursively(fd_pth, base+url+'/')
                out += '</div>'
            else:
                out += '<a href="{{ site.baseurl }}/' + f'{base}{url}">{name}</a>'

        return out
    header += build_the_nested_divs_recursively(tree, '')

    header += (
                '</div>'
            '</div>'
        '</header>'
    )
    
    with open(header_pth, 'w') as file:
        file.write(header)






def rewrite_jekyll_config(pth, author, gh_username, gh_repo):

    text = f"""

# website

baseurl: /{gh_repo}
url: https://{gh_username}.github.io/{gh_repo}


# personal

title: {gh_repo}
author:
  name: {author}


# internal

include: [_pages, _sass, scripts]

permalink: pretty

sass:
  style: compact # possible values: nested expanded compact compressed
  sass_dir: _sass    
"""

    with open(pth, 'w') as f:
        f.write(text)






text = f"""
#-- Welcome to NICS settings!
#----------------------------

#-- Everything starts with "#--" is a comment.
#-- Help: https://nvfp.github.io/now-i-can-sleep


author: 'Nicholas Valentinus'
color_hue: 28
show_credit: True


#-- The variables below should not be changed and are for NICS internal use only.

_gh_username: 'nvfp'
_gh_repo: 'now-i-can-sleep'
"""


import os
import re

from typing import Union, NoReturn


AbsPath = Union[str, os.PathLike]


def _check_format(name):
    res = re.match(r'(?:\d+ -- )?(?P<name>[\w -.]+) -- (?P<url>[\w -]+)(?:.md)?', name)
    if res is None:
        raise

def _inspect_tree_recursively(pth: AbsPath):
    
    for i in os.listdir(pth):
        
        subpth = os.path.join(pth, i)

        if os.path.isdir(subpth):

            ## check dir name format
            res = re.match(r'(?:\d+ -- )?(?P<name>[\w -.]+) -- (?P<url>[\w -]+)', i)
            if res is None:
                raise AssertionError(f'Dir name {repr(i)} format is invalid.')

            _inspect_tree_recursively(subpth)
        else:
            
            ## only .md files are allowed
            if not i.endswith('.md'):
                raise AssertionError(f"File {repr(i)} isn't an .md file.")

            ## check file name format
            if i != 'index.md':
                res = re.match(r'(?:\d+ -- )?(?P<name>[\w -.]+) -- (?P<url>[\w -]+)(?:.md)?', i)
                if res is None:
                    raise AssertionError(f'File name {repr(i)} format is invalid.')

def inspect_the_container(container: AbsPath) -> Union[None, NoReturn]:
    """
    These items are needed inside the container:
    - tree/
    - tree/index.md
    - settings.txt
    """

    ## tree/ folder
    if not os.path.isdir( os.path.join(container, 'tree') ):
        raise AssertionError("Couldn't find 'tree/' in the container.")

    ## homepage
    if not os.path.isfile( os.path.join(container, 'tree', 'index.md') ):
        raise AssertionError("Couldn't find 'tree/index.md' in the container.")

    ## settings file
    if not os.path.isfile( os.path.join(container, 'settings.txt') ):
        raise AssertionError("Couldn't find 'settings.txt' in the container.")

    ## inspecting the tree/
    _inspect_tree_recursively( os.path.join(container, 'tree') )


def inspect_the_dock(dock):
    """shallow inspection to ensure that we are on the correct branch"""

    # if not os.path.isdir( os.path.join(dock, '_includes') ):
    #     raise AssertionError("Folder '_includes' ")